@@ -18,6 +18,17 @@
        (PERL_VERSION > (v) || (PERL_VERSION == (v) && PERL_SUBVERSION >= (s)))))
 #endif
 
+/* C89 bool compatibility - C89 has no bool type */
+#ifndef bool
+   typedef int bool;
+#endif
+#ifndef true
+#  define true 1
+#endif
+#ifndef false
+#  define false 0
+#endif
+
 /* Refcount macros */
 #ifndef SvREFCNT_inc_simple_NN
 #  define SvREFCNT_inc_simple_NN(sv) SvREFCNT_inc(sv)
