@@ -26,7 +26,10 @@
 #  error "object requires Perl 5.10.0 or later"
 #endif
 
-/* C99 bool compatibility */
+/* C89 bool compatibility - C89 has no bool type */
+#ifndef bool
+   typedef int bool;
+#endif
 #ifndef true
 #  define true 1
 #endif
@@ -34,6 +37,17 @@
 #  define false 0
 #endif
 
+/* C89 inline compatibility - C89 has no inline keyword */
+#ifndef OBJECT_INLINE
+#  if defined(__GNUC__) || defined(__clang__)
+#    define OBJECT_INLINE static __inline__
+#  elif defined(_MSC_VER)
+#    define OBJECT_INLINE static __inline
+#  else
+#    define OBJECT_INLINE static
+#  endif
+#endif
+
 /* OP_AELEMFAST_LEX - introduced in 5.16
  * Don't define fallback - code should check if it exists */
 
@@ -129,4 +143,19 @@
 #  define INT2PTR(type, i) ((type)(i))
 #endif
 
+/* GvCV_set - introduced in 5.13.3 (use 5.14 as safe boundary) */
+#ifndef GvCV_set
+#  define GvCV_set(gv, cv) (GvCV(gv) = (cv))
+#endif
+
+/* HvNAMELEN - introduced in 5.16 */
+#ifndef HvNAMELEN
+#  define HvNAMELEN(hv) (HvNAME(hv) ? strlen(HvNAME(hv)) : 0)
+#endif
+
+/* newXS_flags - may not exist on older perls */
+#ifndef newXS_flags
+#  define newXS_flags(name, xsub, file, proto, flags) newXS(name, xsub, file)
+#endif
+
 #endif /* OBJECT_COMPAT_H */
