@@ -19,6 +19,17 @@
        (PERL_VERSION > (v) || (PERL_VERSION == (v) && PERL_SUBVERSION >= (s)))))
 #endif
 
+/* C89 bool compatibility - C89 has no bool type */
+#ifndef bool
+   typedef int bool;
+#endif
+#ifndef true
+#  define true 1
+#endif
+#ifndef false
+#  define false 0
+#endif
+
 /* Op sibling macros - introduced in 5.22 */
 #ifndef OpHAS_SIBLING
 #  define OpHAS_SIBLING(o)      ((o)->op_sibling != NULL)
